@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8; -*-
 
-# Copyright (c) 2021, 2022 Oracle and/or its affiliates.
+# Copyright (c) 2021, 2023 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
 """
@@ -62,6 +62,8 @@ class OptionalDependency:
     GEO = "oracle-ads[geo]"
     ONNX = "oracle-ads[onnx]"
     OPTUNA = "oracle-ads[optuna]"
+    SPARK = "oracle-ads[spark]"
+    HUGGINGFACE = "oracle-ads[huggingface]"
 
 
 def runtime_dependency(
