@@ -1,10 +1,15 @@
 #!/usr/bin/env python
 # -*- coding: utf-8; -*-
 
-# Copyright (c) 2021, 2022 Oracle and/or its affiliates.
+# Copyright (c) 2021, 2023 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
+
+from ads.common import utils as common_utils
 from ads.jobs.builders.base import Builder
 from ads.jobs.builders.runtimes.base import Runtime
+import logging
+
+logger = logging.getLogger(__name__)
 
 
 class Infrastructure(Builder):
@@ -106,6 +111,8 @@ class Infrastructure(Builder):
 
 
 class RunInstance:
+    _DETAILS_LINK = ""
+
     def create(self):
         """Create a RunInstance Object."""
         raise NotImplementedError()
@@ -120,5 +127,34 @@ class RunInstance:
         raise NotImplementedError()
 
     def delete(self):
-        """Delete or cancel a run."""
+        """Delete a run."""
         raise NotImplementedError()
+
+    def cancel(self):
+        """Cancel a run."""
+        raise NotImplementedError()
+
+    @property
+    def run_details_link(self) -> str:
+        """
+        Link to run details page in OCI console
+
+        Returns
+        -------
+        str
+            The link to the details page in OCI console.
+        """
+        if not self._DETAILS_LINK:
+            return ""
+
+        try:
+            return self._DETAILS_LINK.format(
+                region=common_utils.extract_region(), id=self.id
+            )
+        except Exception as ex:
+            print(str(ex))
+            logger.info(
+                "Error occurred in attempt to extract the link to the resource. "
+                f"Details: {ex}"
+            )
+        return ""
