@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8; -*-
 
-# Copyright (c) 2021, 2022 Oracle and/or its affiliates.
+# Copyright (c) 2021, 2023 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 import contextlib
 import logging
@@ -29,13 +29,16 @@ class Artifact:
 
     For example, the NotebookArtifact implements the build() method to convert the notebook to python script.
     with NotebookArtifact(runtime) as artifact:
-        # The build() method will be called when entering the context manager
-        # The final artifact for the job will be stored in artifact.path
-        upload_artifact(artifact.path)
-        # Files are cleaned up when exit or if there is an exception.
+
+    * The build() method will be called when entering the context manager
+    * The final artifact for the job will be stored in artifact.path upload_artifact(artifact.path)
+    * Files are cleaned up when exit or if there is an exception.
 
     """
 
+    CONST_DRIVER_UTILS = "driver_utils.py"
+    CONST_DRIVER_NOTEBOOK = "driver_notebook.py"
+
     def __init__(self, source, runtime=None) -> None:
         # Get the full path of source file if it is local file.
         if not urlparse(source).scheme:
@@ -101,7 +104,7 @@ class Artifact:
 
         Parameters
         ----------
-        uri : URI of the file
+        uri : str
             The URI of the source file or directory, which can be local path of OCI object storage URI.
         to_path : path-like object
             The local destination path.
@@ -118,14 +121,20 @@ class Artifact:
         -------
         str or path-like object
             The actual path of file/directory at destination.
-            For copying a single file and to_path is a filename, this will be the same as to_path.
-            For copying a single file and to_path is a directory, this will be to_path + filename.
-            For copying a directory, this will be to_path + directory name.
+
+            * For copying a single file and to_path is a filename, this will be the same as to_path.
+            * For copying a single file and to_path is a directory, this will be to_path + filename.
+            * For copying a directory, this will be to_path + directory name.
         """
         scheme = urlparse(uri).scheme
         # temp_dir is used only if the uri is zip/tar file
         with tempfile.TemporaryDirectory() as temp_dir:
-            if unpack and (str(uri).endswith("zip") or str(uri).endswith("tar.gz")):
+            if unpack and (
+                str(uri).endswith(".zip")
+                or str(uri).endswith(".tar.gz")
+                or str(uri).endswith(".tar")
+                or str(uri).endswith(".tgz")
+            ):
                 unpack_path = to_path
                 to_path = temp_dir
             else:
@@ -162,34 +171,6 @@ class Artifact:
         raise NotImplementedError()
 
 
-class NotebookArtifact(Artifact):
-    """Represents a NotebookRuntime job artifact"""
-
-    CONST_DRIVER_SCRIPT = "driver_notebook.py"
-
-    def __init__(self, source, runtime) -> None:
-        super().__init__(source, runtime=runtime)
-
-    def build(self):
-        """Prepares job artifact for notebook runtime"""
-        driver_script = os.path.join(
-            os.path.dirname(__file__), "../../templates", self.CONST_DRIVER_SCRIPT
-        )
-        notebook_path = os.path.join(self.temp_dir.name, os.path.basename(self.source))
-        output_path = (
-            os.path.join(
-                self.temp_dir.name,
-                str(os.path.basename(self.source)).split(".", maxsplit=1)[0],
-            )
-            + ".zip"
-        )
-        self.copy_from_uri(self.source, notebook_path)
-        with zipfile.ZipFile(output_path, "w") as zip_file:
-            zip_file.write(notebook_path, os.path.basename(notebook_path))
-            zip_file.write(driver_script, os.path.basename(driver_script))
-        self.path = output_path
-
-
 class ScriptArtifact(Artifact):
     """Represents a ScriptRuntime job artifact"""
 
@@ -209,7 +190,9 @@ class ScriptArtifact(Artifact):
                     "Please specify entrypoint when script source is a directory."
                 )
             output = os.path.join(self.temp_dir.name, basename)
-            shutil.make_archive(output, "zip", os.path.dirname(source), base_dir=basename)
+            shutil.make_archive(
+                output, "zip", os.path.dirname(source), base_dir=basename
+            )
             self.path = output + ".zip"
             return
         # Otherwise, use the artifact directly
@@ -220,31 +203,57 @@ class PythonArtifact(Artifact):
     """Represents a PythonRuntime job artifact"""
 
     CONST_DRIVER_SCRIPT = "driver_python.py"
-    # Temp archive dir storing the zip/tar file from the user
-    ARCHIVE_DIR = "archive"
     # The directory to store user code
     # This directory must match the USER_CODE_DIR in driver_python.py
     USER_CODE_DIR = "code"
 
-    def build(self):
-        """Prepares job artifact for script runtime.
-        If the source is a file, it will be returned as is.
-        If the source is a directory, it will be compressed as a zip file.
-        """
-        driver_script = os.path.join(
-            os.path.dirname(__file__), "../../templates", self.CONST_DRIVER_SCRIPT
-        )
-        basename = os.path.basename(str(self.source).rstrip("/")).split(".", 1)[0]
-        artifact_dir = os.path.join(self.temp_dir.name, basename)
-        code_dir = os.path.join(artifact_dir, self.USER_CODE_DIR)
-        os.makedirs(artifact_dir, exist_ok=True)
+    def __init__(self, source, runtime=None) -> None:
+        super().__init__(source, runtime)
+        self.basename = None
+        self.artifact_dir = None
+        self.code_dir = None
+
+    def _copy_artifacts(self, drivers=None):
+        """Copies the drivers and artifacts to the temp artifact dir."""
+        # The basename of the job artifact,
+        # this will be the name of the zip file uploading to OCI
+        self.basename = os.path.basename(str(self.source).rstrip("/")).split(".", 1)[0]
+        # The temp dir path for storing the artifacts, including drivers and user code
+        self.artifact_dir = os.path.join(self.temp_dir.name, self.basename)
+        # The temp dir path for storing the user code
+        self.code_dir = os.path.join(self.artifact_dir, self.USER_CODE_DIR)
+        os.makedirs(self.artifact_dir, exist_ok=True)
+
+        if not drivers:
+            drivers = []
 
         # Copy the driver script
-        shutil.copy(driver_script, os.path.join(artifact_dir, self.CONST_DRIVER_SCRIPT))
+        for filename in drivers:
+            file_path = os.path.join(
+                os.path.dirname(__file__), "../../templates", filename
+            )
+
+            shutil.copy(file_path, os.path.join(self.artifact_dir, filename))
 
         # Copy user code
-        os.makedirs(code_dir, exist_ok=True)
-        Artifact.copy_from_uri(self.source, code_dir, unpack=True)
+        os.makedirs(self.code_dir, exist_ok=True)
+        Artifact.copy_from_uri(self.source, self.code_dir, unpack=True)
+
+    def _zip_artifacts(self):
+        """Create a zip file from the temp artifact dir."""
+        output = os.path.join(self.temp_dir.name, self.basename)
+        shutil.make_archive(output, "zip", self.artifact_dir, base_dir="./")
+        return output + ".zip"
+
+    def build(self):
+        """Prepares job artifact for PythonRuntime."""
+        self._copy_artifacts(
+            drivers=[
+                self.CONST_DRIVER_UTILS,
+                self.CONST_DRIVER_SCRIPT,
+                self.CONST_DRIVER_NOTEBOOK,
+            ]
+        )
 
         # Check if entrypoint is valid
         # If the user code is a directory,
@@ -254,25 +263,76 @@ class PythonArtifact(Artifact):
         if self.runtime and self.runtime.entrypoint:
             if not os.path.exists(
                 os.path.join(
-                    code_dir, self.runtime.working_dir, self.runtime.entrypoint
+                    self.code_dir, self.runtime.working_dir, self.runtime.entrypoint
                 )
             ):
                 # The specific entrypoint does not exist.
                 # Check if user forgot to specify the top level directory.
                 possible_entrypoint = os.path.join(
-                    code_dir,
+                    self.code_dir,
                     self.runtime.working_dir,
-                    basename,
+                    self.basename,
                     self.runtime.entrypoint,
                 )
                 err_message = (
                     f"Invalid entrypoint. {self.runtime.entrypoint} does not exist."
                 )
                 if os.path.exists(possible_entrypoint):
-                    err_message += f" Do you mean {os.path.join(self.runtime.working_dir, basename, self.runtime.entrypoint)}?"
-                raise ValueError(err_message)
+                    suggested_entrypoint = os.path.join(
+                        self.basename, self.runtime.entrypoint
+                    )
+                    err_message += f" Do you mean {suggested_entrypoint}?"
+                logger.warning(err_message)
 
         # Zip the job artifact
-        output = os.path.join(self.temp_dir.name, basename)
-        shutil.make_archive(output, "zip", artifact_dir, base_dir="./")
-        self.path = output + ".zip"
+        self.path = self._zip_artifacts()
+
+
+class NotebookArtifact(PythonArtifact):
+    """Represents a NotebookRuntime job artifact"""
+
+    CONST_DRIVER_SCRIPT = PythonArtifact.CONST_DRIVER_NOTEBOOK
+
+    def build(self):
+        """Prepares job artifact for notebook runtime"""
+        # Copy job artifacts
+        self._copy_artifacts([self.CONST_DRIVER_UTILS, self.CONST_DRIVER_NOTEBOOK])
+        if self.runtime.notebook:
+            notebook_path = os.path.join(
+                os.path.basename(self.runtime.source), self.runtime.notebook
+            )
+            if not os.path.exists(os.path.join(self.code_dir, notebook_path)):
+                raise ValueError(
+                    f"Invalid notebook path: {self.runtime.notebook}\n"
+                    + f"Please make sure your {self.runtime.source} contains the notebook and "
+                    + "the notebook path is relative to the root of the source."
+                )
+        # Zip the job artifact
+        self.path = self._zip_artifacts()
+
+
+class GitPythonArtifact(Artifact):
+
+    CONST_DRIVER_SCRIPT = "driver_oci.py"
+
+    def __init__(self) -> None:
+        super().__init__("", runtime=None)
+
+    def build(self):
+        """Prepares job artifact for GitPythonRuntime."""
+        artifact_dir = os.path.join(self.temp_dir.name, "ads_driver")
+        os.makedirs(artifact_dir, exist_ok=True)
+        for filename in [
+            self.CONST_DRIVER_UTILS,
+            self.CONST_DRIVER_SCRIPT,
+            NotebookArtifact.CONST_DRIVER_SCRIPT,
+        ]:
+            file_path = os.path.join(
+                os.path.dirname(__file__), "../../templates", filename
+            )
+
+            shutil.copy(file_path, os.path.join(artifact_dir, filename))
+
+        # Zip the job artifact
+        shutil.make_archive(artifact_dir, "zip", artifact_dir, base_dir="./")
+        self.path = artifact_dir + ".zip"
