@@ -1,14 +1,22 @@
 #!/usr/bin/env python
 # -*- coding: utf-8; -*-
 
-# Copyright (c) 2022 Oracle and/or its affiliates.
+# Copyright (c) 2022, 2023 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
+from __future__ import annotations
 import re
-from typing import Dict
+from typing import Dict, TypeVar
 from ads.jobs.builders.base import Builder
 from ads.jobs import env_var_parser
 
 
+Self = TypeVar("Self", bound="Runtime")
+"""Special type to represent the current enclosed class.
+
+This type is used by factory class method or when a method returns ``self``.
+"""
+
+
 class Runtime(Builder):
     """Base class for job runtime"""
 
@@ -34,8 +42,9 @@ class Runtime(Builder):
             self.with_environment_variable(**env)
 
     @property
-    def kind(self):
-        """Kind of the object to be stored in YAML. All runtimes will have "runtime" as kind.
+    def kind(self) -> str:
+        """Kind of the object to be stored in YAML.
+        All runtime implementations will have "runtime" as kind.
         Subclass will have different types.
         """
         return "runtime"
@@ -48,12 +57,10 @@ class Runtime(Builder):
             class_name = re.sub(r"Runtime$", "", class_name)
         return class_name[0].lower() + (class_name[1:] if len(class_name) > 1 else "")
 
-    def with_argument(self, *args, **kwargs):
+    def with_argument(self: Self, *args, **kwargs) -> Self:
         """Adds command line arguments to the runtime.
-        Existing arguments will be preserved.
+
         This method can be called (chained) multiple times to add various arguments.
-        For example, runtime.with_argument(key="val").with_argument("path/to/file") will result in:
-        "--key val path/to/file"
 
         Parameters
         ----------
@@ -68,13 +75,34 @@ class Runtime(Builder):
 
         Returns
         -------
-        Runtime
+        Self
             This method returns self to support chaining methods.
 
         Raises
         ------
         ValueError
             Keyword arguments with space in a key.
+
+        Examples
+        --------
+        >>> runtime = Runtime().with_argument(key1="val1", key2="val2").with_argument("pos1")
+        >>> print(runtime.args)
+        ["--key1", "val1", "--key2", "val2", "pos1"]
+
+        >>> runtime = Runtime()
+        >>> runtime.with_argument("pos1")
+        >>> runtime.with_argument(key1="val1", key2="val2.1 val2.2")
+        >>> runtime.with_argument("pos2")
+        >>> print(runtime.args)
+        ['pos1', '--key1', 'val1', '--key2', 'val2.1 val2.2', 'pos2']
+
+        >>> runtime = Runtime()
+        >>> runtime.with_argument("pos1")
+        >>> runtime.with_argument(key1=None, key2="val2")
+        >>> runtime.with_argument("pos2")
+        >>> print(runtime.args)
+        ["pos1", "--key1", "--key2", "val2", "pos2"]
+
         """
         arg_values = self.get_spec(self.CONST_ARGS, [])
         args = [str(arg) for arg in args]
@@ -90,35 +118,69 @@ class Runtime(Builder):
         self.set_spec(self.CONST_ARGS, arg_values)
         return self
 
-    def with_environment_variable(self, **kwargs):
+    def with_environment_variable(self: Self, **kwargs) -> Self:
         """Sets environment variables
 
+        Environment variables enclosed by ``${...}`` will be substituted.
+
+        * You can use ``$$`` to escape the substitution.
+        * Undefined variable enclosed by ``${}`` will be ignored.
+        * Double dollar signs ``$$`` will be substituted by a single one ``$``.
+
         Returns
         -------
-        Runtime
+        Self
             This method returns self to support chaining methods.
+
+        Examples
+        --------
+        >>> runtime = (
+        ...     PythonRuntime()
+        ...     .with_environment_variable(
+        ...         HOST="10.0.0.1",
+        ...         PORT="443",
+        ...         URL="http://${HOST}:${PORT}/path/",
+        ...         ESCAPED_URL="http://$${HOST}:$${PORT}/path/",
+        ...         MISSING_VAR="This is ${UNDEFINED}",
+        ...         VAR_WITH_DOLLAR="$10",
+        ...         DOUBLE_DOLLAR="$$10"
+        ...     )
+        ... )
+        >>> for k, v in runtime.environment_variables.items():
+        ...     print(f"{k}: {v}")
+        HOST: 10.0.0.1
+        PORT: 443
+        URL: http://10.0.0.1:443/path/
+        ESCAPED_URL: http://${HOST}:${PORT}/path/
+        MISSING_VAR: This is ${UNDEFINED}
+        VAR_WITH_DOLLAR: $10
+        DOUBLE_DOLLAR: $10
+
+
         """
         if not kwargs:
             return self
         envs = [{"name": k, "value": v} for k, v in kwargs.items()]
         return self.set_spec(self.CONST_ENV_VAR, envs)
 
-    def with_freeform_tag(self, **kwargs):
+    def with_freeform_tag(self: Self, **kwargs) -> Self:
         """Sets freeform tag
 
         Returns
         -------
-        Runtime
+        Self
             This method returns self to support chaining methods.
         """
         return self.set_spec(self.CONST_TAG, kwargs)
 
-    def with_maximum_runtime_in_minutes(self, maximum_runtime_in_minutes: int):
+    def with_maximum_runtime_in_minutes(
+        self: Self, maximum_runtime_in_minutes: int
+    ) -> Self:
         """Sets maximum runtime in minutes
 
         Returns
         -------
-        Runtime
+        Self
             This method returns self to support chaining methods.
         """
         return self.set_spec(
