@@ -1,13 +1,13 @@
 #!/usr/bin/env python
 # -*- coding: utf-8; -*-
 
-# Copyright (c) 2021, 2022 Oracle and/or its affiliates.
+# Copyright (c) 2021, 2023 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 from __future__ import annotations
 
 import json
 import os
-from typing import Any
+from typing import Any, Dict
 
 from ads.jobs.builders.runtimes.base import Runtime
 from ads.common.auth import default_signer
@@ -15,7 +15,10 @@ from ads.opctl.config.utils import convert_notebook
 
 
 class CondaRuntime(Runtime):
-    """Represents a job runtime with conda pack"""
+    """Represents a job runtime with conda pack
+    This is the base class for Runtime using conda environment.
+    The ``CondaRuntime`` is not designed to be used directly when creating a job.
+    """
 
     CONST_CONDA = "conda"
     CONST_CONDA_TYPE = "type"
@@ -30,12 +33,28 @@ class CondaRuntime(Runtime):
 
     @property
     def conda(self) -> dict:
-        """The conda pack specification
+        """The conda environment specification.
+
+        For service conda environment, the specification contains:
+
+        * ``type``, the type of the conda environment.
+          This is always ``service`` for service conda environment.
+        * ``slug``, the slug of the conda environment.
+
+        For custom conda environment, the specification contains:
+
+        * ``type``, the type of the conda environment.
+          This is always ``published`` for custom conda environment.
+        * ``uri``, the uri of the conda environment, e.g. oci://bucket@namespace/prefix/to/conda
+        * ``region``, the region of the bucket in which the conda environment is stored.
+          By default, ADS will determine the region based on the authenticated API key or resource principal.
+          This is only needed if your conda environment is stored in a different region.
 
         Returns
         -------
         dict
-            A dictionary with "type" and "slug" as keys.
+            A dictionary containing the conda environment specifications.
+
         """
         return self.get_spec(self.CONST_CONDA)
 
@@ -62,6 +81,7 @@ class CondaRuntime(Runtime):
 
     def with_custom_conda(self, uri: str, region: str = None):
         """Specifies the custom conda pack for running the job
+        Make sure you have configured the IAM policy for the job run to access the conda environment.
 
         Parameters
         ----------
@@ -72,7 +92,8 @@ class CondaRuntime(Runtime):
             this is shown as the "source" of the conda pack.
         region: str, optional
             The region of the bucket storing the custom conda pack, by default None.
-            If region is not specified, ADS will use the region from your authentication credentials,
+            If region is not specified, ADS will use the region from your authentication credentials:
+
             * For API Key, config["region"] is used.
             * For Resource Principal, signer.region is used.
 
@@ -98,7 +119,36 @@ class CondaRuntime(Runtime):
 
 
 class ScriptRuntime(CondaRuntime):
-    """Represents job runtime with scripts and conda pack"""
+    """Represents job runtime with scripts and conda pack.
+
+    This runtime is designed to define job artifacts and configurations supported by OCI Data Science Jobs natively.
+    It can be used with any script types that is supported by the OCI Data Science Jobs,
+    including shell scripts and python scripts.
+
+    To run a script with all dependencies contained in a local folder::
+
+        runtime = (
+            ScriptRuntime()
+            # Specify the service conda environment by slug name.
+            .with_service_conda("pytorch110_p38_cpu_v1")
+            # The job artifact can be a single Python script, a directory or a zip file.
+            .with_source("local/path/to/code_dir")
+            # Environment variable
+            .with_environment_variable(NAME="Welcome to OCI Data Science.")
+            # Command line argument
+            .with_argument("100 linux \"hi there\"")
+            # The entrypoint is applicable only to directory or zip file as source
+            # The entrypoint should be a path relative to the working dir.
+            # Here my_script.sh is a file in the code_dir/my_package directory
+            .with_entrypoint("my_package/my_script.sh")
+        )
+
+
+    References
+    ----------
+    https://docs.oracle.com/en-us/iaas/data-science/using/jobs-artifact.htm
+
+    """
 
     CONST_ENTRYPOINT = "entrypoint"
     CONST_SCRIPT_PATH = "scriptPathURI"
@@ -120,7 +170,7 @@ class ScriptRuntime(CondaRuntime):
         Parameters
         ----------
         uri : str
-            URI to the Python or Shell script, which can be any URI supported by fsspec,
+            URI to the source code script, which can be any URI supported by fsspec,
             including http://, https:// and OCI object storage.
             For example: oci://your_bucket@your_namespace/path/to/script.py
 
@@ -147,7 +197,8 @@ class ScriptRuntime(CondaRuntime):
             If the source code is a single file, URI can be any URI supported by fsspec,
             including http://, https:// and OCI object storage.
             For example: oci://your_bucket@your_namespace/path/to/script.py
-            If the source code is a directory, only local directory is supported.
+            URI can also be a folder or a zip file containing the source code.
+            In that case, entrypoint is required.
 
         entrypoint : str, optional
             The relative path of the script to be set as entrypoint when source is a zip/tar/directory.
@@ -189,6 +240,7 @@ class _PythonRuntimeMixin(Runtime):
     CONST_PYTHON_PATH = "pythonPath"
     CONST_ENTRYPOINT = "entrypoint"
     CONST_ENTRY_FUNCTION = "entryFunction"
+    CONST_WORKING_DIR = "workingDir"
 
     attribute_map = {
         CONST_OUTPUT_DIR: "output_dir",
@@ -196,6 +248,7 @@ class _PythonRuntimeMixin(Runtime):
         CONST_PYTHON_PATH: "python_path",
         CONST_ENTRYPOINT: CONST_ENTRYPOINT,
         CONST_ENTRY_FUNCTION: "entry_function",
+        CONST_WORKING_DIR: "working_dir",
     }
     attribute_map.update(Runtime.attribute_map)
 
@@ -215,7 +268,7 @@ class _PythonRuntimeMixin(Runtime):
 
         Returns
         -------
-        self
+        Self
             The runtime instance.
         """
         self.set_spec(self.CONST_OUTPUT_DIR, output_dir)
@@ -266,6 +319,29 @@ class _PythonRuntimeMixin(Runtime):
         self.set_spec(self.CONST_ENTRY_FUNCTION, func)
         return self
 
+    def with_working_dir(self, working_dir: str):
+        """Specifies the working directory in the job run.
+        By default, the working directory will the directory containing the user code (job artifact directory).
+        This can be changed by specifying a relative path to the job artifact directory.
+
+        Parameters
+        ----------
+        working_dir : str
+            The path of the working directory.
+            This can be a relative path from the job artifact directory.
+
+        Returns
+        -------
+        self
+            The runtime instance.
+        """
+        return self.set_spec(self.CONST_WORKING_DIR, working_dir)
+
+    @property
+    def working_dir(self) -> str:
+        """The working directory for the job run."""
+        return self.get_spec(self.CONST_WORKING_DIR, ".")
+
     @property
     def output_dir(self) -> str:
         """Directory in the Job run container for saving output files generated in the job"""
@@ -293,53 +369,91 @@ class _PythonRuntimeMixin(Runtime):
 
 
 class PythonRuntime(ScriptRuntime, _PythonRuntimeMixin):
-    """Represents a job runtime using ADS driver script to run Python code"""
+    """Represents a job runtime using ADS driver script to run Python code
 
-    CONST_WORKING_DIR = "workingDir"
-    attribute_map = {CONST_WORKING_DIR: "working_dir"}
+    Example::
+
+        runtime = (
+            PythonRuntime()
+            # Specify the service conda environment by slug name.
+            .with_service_conda("pytorch110_p38_cpu_v1")
+            # The job artifact can be a single Python script, a directory or a zip file.
+            .with_source("local/path/to/code_dir")
+            # Environment variable
+            .with_environment_variable(NAME="Welcome to OCI Data Science.")
+            # Command line argument, arg1 --key arg2
+            .with_argument("arg1", key="arg2")
+            # Set the working directory
+            # When using a directory as source, the default working dir is the parent of code_dir.
+            # Working dir should be a relative path beginning from the source directory (code_dir)
+            .with_working_dir("code_dir")
+            # The entrypoint is applicable only to directory or zip file as source
+            # The entrypoint should be a path relative to the working dir.
+            # Here my_script.py is a file in the code_dir/my_package directory
+            .with_entrypoint("my_package/my_script.py")
+            # Add an additional Python path, relative to the working dir (code_dir/other_packages).
+            .with_python_path("other_packages")
+            # Copy files in "code_dir/output" to object storage after job finishes.
+            .with_output("output", "oci://bucket_name@namespace/path/to/dir")
+        )
+
+    """
+
+    attribute_map = {}
     attribute_map.update(ScriptRuntime.attribute_map)
     attribute_map.update(_PythonRuntimeMixin.attribute_map)
 
-    def with_working_dir(self, working_dir: str):
-        """Specifies the working directory in the job run.
-        By default, the working directory will the directory containing the user code (job artifact directory).
-        This can be changed by specifying a relative path to the job artifact directory.
-
-        Parameters
-        ----------
-        working_dir : str
-            The path of the working directory.
-            This can be a relative path from the job artifact directory.
-
-        Returns
-        -------
-        self
-            The runtime instance.
-        """
-        return self.set_spec(self.CONST_WORKING_DIR, working_dir)
-
-    @property
-    def working_dir(self) -> str:
-        """The working directory for the job run."""
-        return self.get_spec(self.CONST_WORKING_DIR, ".")
-
 
 class NotebookRuntime(CondaRuntime):
-    """Represents a job runtime with Jupyter notebook"""
+    """Represents a job runtime with Jupyter notebook
+
+    To run a job with a single Jupyter notebook,
+    you can define the run time as::
+
+        runtime = (
+            NotebookRuntime()
+            .with_notebook(
+                path="https://raw.githubusercontent.com/tensorflow/docs/master/site/en/tutorials/customization/basics.ipynb",
+                encoding='utf-8'
+            )
+            .with_service_conda("tensorflow28_p38_cpu_v1")
+            .with_environment_variable(GREETINGS="Welcome to OCI Data Science")
+            .with_exclude_tag(["ignore", "remove"])
+            .with_output("oci://bucket_name@namespace/path/to/dir")
+        )
+
+    Note that the notebook path can be local or remote path supported by fsspec,
+    including OCI object storage path like ``oci://bucket@namespace/path/to/notebook``
+
+    """
 
     CONST_NOTEBOOK_PATH = "notebookPathURI"
     CONST_NOTEBOOK_ENCODING = "notebookEncoding"
-    CONST_OUTPUT_URI = "outputURI"
+    CONST_OUTPUT_URI = "outputUri"
+    CONST_OUTPUT_URI_ALT = "outputURI"
     CONST_EXCLUDE_TAG = "excludeTags"
+    CONST_SOURCE = "source"
+    CONST_ENTRYPOINT = "entrypoint"
 
     attribute_map = {
         CONST_NOTEBOOK_PATH: "notebook_path_uri",
         CONST_NOTEBOOK_ENCODING: "notebook_encoding",
         CONST_OUTPUT_URI: "output_uri",
         CONST_EXCLUDE_TAG: "exclude_tags",
+        CONST_SOURCE: "source",
+        CONST_ENTRYPOINT: "entrypoint",
     }
     attribute_map.update(CondaRuntime.attribute_map)
 
+    def __init__(self, spec: Dict = None, **kwargs) -> None:
+        if spec and self.CONST_OUTPUT_URI_ALT in spec:
+            val = spec.pop(self.CONST_OUTPUT_URI_ALT)
+            spec[self.CONST_OUTPUT_URI] = val
+        if self.CONST_OUTPUT_URI_ALT in kwargs:
+            val = kwargs.pop(self.CONST_OUTPUT_URI_ALT)
+            kwargs[self.CONST_OUTPUT_URI] = val
+        super().__init__(spec, **kwargs)
+
     @property
     def notebook_uri(self) -> str:
         """The URI of the notebook"""
@@ -350,13 +464,17 @@ class NotebookRuntime(CondaRuntime):
         """The encoding of the notebook"""
         return self.get_spec(self.CONST_NOTEBOOK_ENCODING)
 
-    def with_notebook(self, path: str, encoding="utf-8"):
-        """Specifies the notebook to be converted to python script and run as a job.
+    def with_notebook(self, path: str, encoding="utf-8") -> NotebookRuntime:
+        """Specifies the notebook to be run as a job.
+        Use this method if you would like to run a single notebook.
+        Use ``with_source()`` method if you would like to run a notebook with additional dependency files.
 
         Parameters
         ----------
         path : str
             The path of the Jupyter notebook
+        encoding : str
+            The encoding for opening the notebook. Defaults to utf-8.
 
         Returns
         -------
@@ -371,7 +489,7 @@ class NotebookRuntime(CondaRuntime):
         """A list of cell tags indicating cells to be excluded from the job"""
         return self.get_spec(self.CONST_EXCLUDE_TAG, [])
 
-    def with_exclude_tag(self, *tags):
+    def with_exclude_tag(self, *tags) -> NotebookRuntime:
         """Specifies the cell tags in the notebook to exclude cells from the job script.
 
         Parameters
@@ -397,13 +515,14 @@ class NotebookRuntime(CondaRuntime):
         """URI for storing the output notebook and files"""
         return self.get_spec(self.CONST_OUTPUT_URI)
 
-    def with_output(self, output_uri: str):
+    def with_output(self, output_uri: str) -> NotebookRuntime:
         """Specifies the output URI for storing the output notebook and files.
+        All files in the directory containing the notebook will be saved.
 
         Parameters
         ----------
         output_uri : str
-            URI for storing the output notebook and files.
+            URI for a directory storing the output notebook and files.
             For example, oci://bucket@namespace/path/to/dir
 
         Returns
@@ -413,9 +532,70 @@ class NotebookRuntime(CondaRuntime):
         """
         return self.set_spec(self.CONST_OUTPUT_URI, output_uri)
 
+    def with_source(self, uri: str, notebook: str, encoding="utf-8"):
+        """Specify source code directory containing the notebook and dependencies for the job.
+        Use this method if you would like to run a notebook with additional dependency files.
+        Use the `with_notebook()` method if you would like to run a single notebook.
+
+        In the following example, local folder "path/to/source" contains the notebook and dependencies,
+        The local path of the notebook is "path/to/source/relative/path/to/notebook.ipynb"::
+
+            runtime.with_source(uri="path/to/source", notebook="relative/path/to/notebook.ipynb")
+
+        Parameters
+        ----------
+        uri : str
+            URI of the source code directory. This can be local or on OCI object storage.
+        notebook : str
+            The relative path of the notebook from the source URI.
+        encoding : str
+            The encoding for opening the notebook. Defaults to utf-8.
+
+        Returns
+        -------
+        Self
+            The runtime instance.
+
+        """
+        self.set_spec(self.CONST_SOURCE, uri)
+        self.set_spec(self.CONST_ENTRYPOINT, notebook)
+        self.set_spec(self.CONST_NOTEBOOK_ENCODING, encoding)
+        return self
+
+    @property
+    def source(self) -> str:
+        """The source code location."""
+        return self.get_spec(self.CONST_SOURCE)
+
+    @property
+    def notebook(self) -> str:
+        """The path of the notebook relative to the source."""
+        return self.get_spec(self.CONST_ENTRYPOINT)
+
 
 class GitPythonRuntime(CondaRuntime, _PythonRuntimeMixin):
-    """Represents a job runtime with source code from git repository"""
+    """Represents a job runtime with source code from git repository
+
+    Example::
+
+        runtime = (
+            GitPythonRuntime()
+            .with_environment_variable(GREETINGS="Welcome to OCI Data Science")
+            # Specify the service conda environment by slug name.
+            .with_service_conda("pytorch19_p37_gpu_v1")
+            # Specify the git repository
+            # Optionally, you can specify the branch or commit
+            .with_source("https://github.com/pytorch/tutorials.git")
+            # Entrypoint is a relative path from the root of the git repo.
+            .with_entrypoint("beginner_source/examples_nn/polynomial_nn.py")
+            # Copy files in "beginner_source/examples_nn" to object storage after job finishes.
+            .with_output(
+              output_dir="beginner_source/examples_nn",
+              output_uri="oci://bucket_name@namespace/path/to/dir"
+            )
+        )
+
+    """
 
     CONST_GIT_URL = "url"
     CONST_BRANCH = "branch"
@@ -494,54 +674,8 @@ class GitPythonRuntime(CondaRuntime, _PythonRuntimeMixin):
         """Git commit ID (SHA1 hash)"""
         return self.get_spec(self.CONST_COMMIT)
 
-    @staticmethod
-    def _serialize_arg(arg: Any) -> str:
-        """Serialize the argument.
-        This returns the argument "as is" if it is a string AND not a valid JSON payload.
-        Otherwise the argument will be serialized with JSON.
-
-        Parameters
-        ----------
-        arg : Any
-            argument to be serialized
-
-        Returns
-        -------
-        str
-            Serialized argument as a string
-        """
-        if arg is None:
-            return None
-        if isinstance(arg, str):
-            try:
-                json.loads(arg)
-            except json.JSONDecodeError:
-                return arg
-        return json.dumps(arg)
-
-    def with_argument(self, *args, **kwargs):
-        """Specifies the arguments for running the script/function.
-
-        When running a python script, the arguments will be the command line arguments.
-        For example, with_argument("arg1", "arg2", key1="val1", key2="val2")
-        will generate the command line arguments: "arg1 arg2 --key1 val1 --key2 val2"
-
-        When running a function, the arguments will be passed into the function.
-        Arguments can also be list, dict or any JSON serializable object.
-        For example, with_argument("arg1", "arg2", key1=["val1a", "val1b"], key2="val2")
-        will be passed in as "your_function("arg1", "arg2", key1=["val1a", "val1b"], key2="val2")
-
-        Returns
-        -------
-        self
-            The runtime instance.
-        """
-        args = [self._serialize_arg(arg) for arg in args]
-        kwargs = {k: self._serialize_arg(v) for k, v in kwargs.items()}
-        return super().with_argument(*args, **kwargs)
-
     @property
-    def ssh_secret_ocid(self):
+    def ssh_secret_ocid(self) -> str:
         """The OCID of the OCI Vault secret storing the Git SSH key."""
         return self.get_spec(self.CONST_GIT_SSH_SECRET_ID)
 
@@ -554,6 +688,7 @@ class DataFlowRuntime(CondaRuntime):
     CONST_SCRIPT_PATH = "scriptPathURI"
     CONST_CONFIGURATION = "configuration"
     CONST_CONDA_AUTH_TYPE = "condaAuthType"
+    CONST_OVERWRITE = "overwrite"
     attribute_map = {
         CONST_SCRIPT_BUCKET: "script_bucket",
         CONST_ARCHIVE_URI: "archive_bucket",
@@ -561,6 +696,7 @@ class DataFlowRuntime(CondaRuntime):
         CONST_SCRIPT_PATH: "script_path_uri",
         CONST_CONFIGURATION: CONST_CONFIGURATION,
         CONST_CONDA_AUTH_TYPE: "conda_auth_type",
+        CONST_OVERWRITE: CONST_OVERWRITE,
     }
     attribute_map.update(Runtime.attribute_map)
 
@@ -645,13 +781,13 @@ class DataFlowRuntime(CondaRuntime):
         """The URI of the source code"""
         return self.get_spec(self.CONST_SCRIPT_PATH)
 
-    def with_script_uri(self, path) -> "DataFlowRuntime":
+    def with_script_uri(self, path: str) -> "DataFlowRuntime":
         """
         Set script uri.
 
         Parameters
         ----------
-        uri: str
+        path: str
             uri to the script
 
         Returns
@@ -726,6 +862,29 @@ class DataFlowRuntime(CondaRuntime):
         """Configuration for Spark"""
         return self.get_spec(self.CONST_CONFIGURATION)
 
+    def with_overwrite(self, overwrite: bool) -> "DataFlowRuntime":
+        """
+        Whether to overwrite the existing script in object storage (script bucket).
+        If the Object Storage bucket already contains a script with the same name,
+        then it will be overwritten with the new one if the `overwrite` flag equal to `True`.
+
+        Parameters
+        ----------
+        overwrite: bool
+            Whether to overwrite the existing script in object storage (script bucket).
+
+        Returns
+        -------
+        DataFlowRuntime
+             The DataFlowRuntime instance (self).
+        """
+        return self.set_spec(self.CONST_OVERWRITE, overwrite)
+
+    @property
+    def overwrite(self) -> str:
+        """Whether to overwrite the existing script in object storage (script bucket)."""
+        return self.get_spec(self.CONST_OVERWRITE)
+
     def convert(self, **kwargs):
         pass
 
