@@ -1,8 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*--
 
-# Copyright (c) 2021, 2022 Oracle and its affiliates.
-# Copyright (c) 2021, 2022 Oracle and/or its affiliates.
+# Copyright (c) 2021, 2023 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
 from .model_deployer import ModelDeployer
@@ -10,5 +9,12 @@ from .model_deployment import (
     DEFAULT_POLL_INTERVAL,
     DEFAULT_WAIT_TIME,
     ModelDeployment,
+    ModelDeploymentMode,
 )
 from .model_deployment_properties import ModelDeploymentProperties
+from .model_deployment_infrastructure import ModelDeploymentInfrastructure
+from .model_deployment_runtime import (
+    ModelDeploymentRuntime,
+    ModelDeploymentCondaRuntime,
+    ModelDeploymentContainerRuntime,
+)
