@@ -5,7 +5,11 @@
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
 from ads.model.extractor.model_info_extractor import ModelInfoExtractor
-from ads.common.model_metadata import Framework
+from ads.model.model_metadata import Framework
+from ads.common.decorator.runtime_dependency import (
+    runtime_dependency,
+    OptionalDependency,
+)
 
 
 class TensorflowExtractor(ModelInfoExtractor):
@@ -56,6 +60,9 @@ class TensorflowExtractor(ModelInfoExtractor):
         return self.model.__class__.__name__
 
     @property
+    @runtime_dependency(
+        module="tensorflow", short_name="tf", install_from=OptionalDependency.TENSORFLOW
+    )
     def version(self):
         """Extracts the framework version of the model.
 
@@ -64,9 +71,7 @@ class TensorflowExtractor(ModelInfoExtractor):
         str:
            The framework version of the model.
         """
-        import tensorflow
-
-        return tensorflow.__version__
+        return tf.__version__
 
     @property
     def hyperparameter(self):
