@@ -6,6 +6,7 @@
 
 import click
 
+from ads.common.auth import AuthType
 from ads.opctl.conda.cmds import create as create_cmd
 from ads.opctl.conda.cmds import install as install_cmd
 from ads.opctl.conda.cmds import publish as publish_cmd
@@ -54,6 +55,12 @@ def commands():
     is_flag=True,
     default=False,
 )
+@click.option(
+    "--verbose",
+    is_flag=True,
+    default=False,
+    help="Set logging to debug for more information",
+)
 @click.help_option("--help", "-h")
 @click.option("--debug", "-d", help="set debug mode", is_flag=True, default=False)
 def create(**kwargs):
@@ -106,7 +113,7 @@ def create(**kwargs):
     "--auth",
     "-a",
     help="authentication method",
-    type=click.Choice(["api_key", "resource_principal"]),
+    type=click.Choice(AuthType.values()),
     default=None,
 )
 def install(**kwargs):
@@ -178,7 +185,7 @@ def install(**kwargs):
     "--auth",
     "-a",
     help="authentication method",
-    type=click.Choice(["api_key", "resource_principal"]),
+    type=click.Choice(AuthType.values()),
     default=None,
 )
 def publish(**kwargs):
